@@ -1,10 +1,10 @@
 /*
- * Copyright 2005-2007 Gentoo Foundation
+ * Copyright 2005-2010 Gentoo Foundation
  * Distributed under the terms of the GNU General Public License v2
- * $Header: /var/cvsroot/gentoo-projects/portage-utils/qmerge.c,v 1.92 2010/01/13 19:16:22 vapier Exp $
+ * $Header: /var/cvsroot/gentoo-projects/portage-utils/qmerge.c,v 1.95 2010/06/08 04:52:42 vapier Exp $
  *
- * Copyright 2005-2007 Ned Ludd        - <solar@gentoo.org>
- * Copyright 2005-2007 Mike Frysinger  - <vapier@gentoo.org>
+ * Copyright 2005-2010 Ned Ludd        - <solar@gentoo.org>
+ * Copyright 2005-2010 Mike Frysinger  - <vapier@gentoo.org>
  */
 
 #ifdef APPLET_qmerge
@@ -55,7 +55,7 @@ static const char *qmerge_opts_help[] = {
 	COMMON_OPTS_HELP
 };
 
-static const char qmerge_rcsid[] = "$Id: qmerge.c,v 1.92 2010/01/13 19:16:22 vapier Exp $";
+static const char qmerge_rcsid[] = "$Id: qmerge.c,v 1.95 2010/06/08 04:52:42 vapier Exp $";
 #define qmerge_usage(ret) usage(ret, QMERGE_FLAGS, qmerge_long_opts, qmerge_opts_help, lookup_applet_idx("qmerge"))
 
 char search_pkgs = 0;
@@ -402,6 +402,22 @@ char qprint_tree_node(int level, depend_atom *atom, struct pkg_t *pkg)
 	return c;
 }
 
+static int has_postinst(char *vdbroot)
+{
+	FILE *fp;
+	int ret;
+	char buf[1024];
+	snprintf(buf, sizeof(buf), "%s/%s", vdbroot, "DEFINED_PHASES");
+	if ((fp = fopen(buf,  "r")) == NULL)
+		return 1;
+	if (fgets(buf, sizeof(buf), fp))
+		ret = (strstr(buf, "postinst") == NULL) ? 0 : 2;
+	else
+		ret = 0;
+	fclose(fp);
+	return ret;
+}
+
 /* oh shit getting into pkg mgt here. FIXME: write a real dep resolver. */
 void pkg_merge(int level, depend_atom *atom, struct pkg_t *pkg)
 {
@@ -784,7 +800,7 @@ void pkg_merge(int level, depend_atom *atom, struct pkg_t *pkg)
 	interactive_rename("vdb", buf, pkg);
 
 	/* run postinst on non embedded systems */
-	if (which("ebuild") != NULL) {
+	if ((which("ebuild") != NULL) && has_postinst(buf)) {
 		char *tbuf;
 		xasprintf(&tbuf, "ebuild %s/%s.ebuild postinst", buf, basename(buf));
 		xsystem(tbuf);
